local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

local AceEvent = LibStub:GetLibrary("AceEvent-3.0")

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- the Inlets module
local Inlets = Addon:NewModule("Inlets", "AceEvent-3.0")

-- local functions
local tinsert           = table.insert
local pairs             = pairs

-- inlet
NS.INLET  = "inlet"

local Inlet_Prototype = {}

-- inlet functions
function Inlet_Prototype:UpdateActive(event, name)
	Addon:Debug(self:GetName() .. " UpdateActive: " .. tostring(name))

	if name == self:GetName() then
		self:UpdateNodeSelection()
		Addon:Debug(self:GetName() .. " UpdateActive: register MAGNETO_NODE_TYPES_CHANGED")
		self:RegisterMessage("MAGNETO_NODE_TYPES_CHANGED", "UpdateNodeSelection")
	else
		Addon:Debug(self:GetName() .. " UpdateActive: unregister MAGNETO_NODE_TYPES_CHANGED")
		self:UnregisterMessage("MAGNETO_NODE_TYPES_CHANGED")
	end
end

function Inlet_Prototype:GetNodesIterator(zone, profession)
	Addon:Debug(self:GetName() .. " GetNodesIterator: empty prototype")
	if not self:IsReady() then
		return
	end
	
	-- empty dummy, overwrite in specific implementation
end

function Inlet_Prototype:UpdateNodeSelection(event)
	Addon:Debug(self:GetName() .. " UpdateNodeSelection: empty prototype")
	if not self:IsReady() then
		return
	end
	
	-- called selection of tracked node types has changed
	-- empty dummy, overwrite in specific implementation
end

-- factory method for inlet creation
function NS:CreateInlet(name, description)
	local module = NS:CreateModule(NS.INLET, name, description)
	
	module.nodes = {}
	
	for k, v in pairs(Inlet_Prototype) do
		module[k] = v
	end
	
	AceEvent:Embed(module)
	
	module:RegisterMessage("MAGNETO_INLET_ACTIVE_CHANGED", "UpdateActive")
	
	return module
end

function NS:IsValidInlet(module)
	if not NS:IsValidModule(module) or module:GetType() ~= NS.INLET then
		return false
	end

	for k, _ in pairs(Inlet_Prototype) do
		if not module[k] then
			return false
		end
	end
	
	return true
end

Inlets.NONE = "none"

-- module data
local moduleData = {
	inlets       = {},
	activeName   = "none",
}

-- inlet handling
function Inlets:OnInitialize()
	-- empty
end

function Inlets:OnEnable()
	self:RegisterInlets()
end

function Inlets:OnDisable()
	self:UnregisterInlets()
end

function Inlets:RegisterInlets()
	for name, inlet in NS:IterateModulesByType(NS.INLET) do
		self:RegisterInlet(inlet)
	end
end

function Inlets:RegisterInlet(inlet)
	if not inlet then 
		self:Debug("RegisterInlet: no inlet provided")
		return false
	end
	
	-- check interface
	if not NS:IsValidInlet(inlet)	then
		self:Debug("RegisterInlet: inlet doesn't implement interface")
		return false
	end
	
	local name = inlet:GetName()
	
	if not name or name == "" then
		self:Debug("RegisterInlet: inlet doesn't provide name")
		return false
	end

	if name == self.NONE then
		self:Debug("RegisterInlet: inlet provides reserved name '" .. self.NONE .. "'")
		return false
	end
	
	-- already registered
	if moduleData.inlets[name] then
		self:Debug("RegisterInlet: inlet '" .. tostring(name) .. "' already registered")
		return false
	end
	
	-- register inlet
	moduleData.inlets[name] = inlet
	
	self:SendMessage("MAGNETO_INLET_REGISTERED_CHANGED", name, inlet, true)
	
	return true
end

function Inlets:UnregisterInlets()
	for name, inlet in pairs(moduleData.inlets) do
		self:UnregisterInlet(inlet)
	end
end

function Inlets:UnregisterInlet(inlet)
	if not inlet or not inlet.GetName or not self:HasInlet(inlet:GetName()) then 
		return
	end
	
	local name = inlet:GetName()
	
	-- deactivate
	if name == self:GetActiveInletName() then
		self:SetActiveInlet(self.NONE)
	end
	
	-- unregister inlet
	moduleData.inlets[name] = nil
	
	self:SendMessage("MAGNETO_INLET_REGISTERED_CHANGED", name, inlet, false)	
end

function Inlets:HasInlet(name)
	return moduleData.inlets[name] and true or false
end

function Inlets:GetInlet(name)
	return moduleData.inlets[name]
end

function Inlets:IsInletOperational()
	local inlet = self:GetActiveInlet()
	
	return inlet and inlet:IsReady() or false
end

function Inlets:IsInletReady(name)
	return self:HasInlet(name) and moduleData.inlets[name]:IsReady() or false
end

function Inlets:IsInletActive(name)
	return self:GetActiveInletName() == name
end

function Inlets:SetActiveInlet(name)
	if not self:HasInlet(name) and name ~= self.NONE then
		return
	end

	if moduleData.activeName == name then
		return
	end
	
	moduleData.activeName = name
	
	self:SendMessage("MAGNETO_INLET_ACTIVE_CHANGED", name)
end

function Inlets:GetActiveInletName()
	return moduleData.activeName
end

function Inlets:GetActiveInlet()
	return self:GetInlet(moduleData.activeName)
end

function Inlets:IterateInlets()
	return pairs(moduleData.inlets)
end

-- test
function Inlets:Debug(msg)
	Addon:Debug("(Inlets) " .. msg)
end
